pub mod entities;
use entities::{prelude::*,*};
pub use sea_orm::*;
use rand::Rng;


pub fn prepare_types_for_print(types: Vec<entities::types::Model>) -> String {
    let prepared_vector: Vec<String>= types.into_iter().map(
        |item: entities::types::Model| {
            format!(
                "{:}: {:}",
                item.id,
                item.title
            )
        }
    ).collect();
    prepared_vector.join("\n")
}

pub async fn create_video(
    db: &DatabaseConnection,
    title: &str, type_id: i32, url: &str, duration: i32
) -> Result<InsertResult<videos::ActiveModel>, DbErr> {
    let video = entities::videos::ActiveModel {
        title: ActiveValue::Set(title.to_owned()),
        type_id: ActiveValue::Set(
            type_id
        ),
        url: ActiveValue::Set(
            url.to_owned()
        ),
        duration: ActiveValue::Set(Some(duration)),
        ..Default::default()
    };
    let res: Result<InsertResult<videos::ActiveModel>, DbErr> =
        Videos::insert(video).exec(db).await;
    println!("{:?}", res);
    res
}

pub async fn get_random_video_entry_of_type(db: &DatabaseConnection, type_id: i32) -> Result<videos::Model, DbErr> {

    let videos_vec: Vec<entities::videos::Model> = videos::Entity::find()
        .filter(videos::Column::TypeId.eq(type_id)).all(db).await?;

    if videos_vec.len() <= 0 {
        return Err(DbErr::RecordNotFound(String::from("Нет элементов с заданым типом")))
    }

    let res_vid = match videos_vec.get(rand::thread_rng().gen_range(0..videos_vec.len())){
        Some(val) => val,
        None => return Err(DbErr::RecordNotFound(String::from("Нет элементов с заданым типом")))
    };

    let accsess_count = res_vid.access_count.unwrap_or_default();

    let mut u_res_vid: videos::ActiveModel = res_vid.clone().into_active_model();
    u_res_vid.access_count = ActiveValue::Set(Some(accsess_count + 1));

    u_res_vid.update(db).await?;

    Ok(res_vid.to_owned())

}

pub async fn get_shitpost_types(
    db: &DatabaseConnection
) -> Result<Vec<entities::types::Model>, DbErr> {
    entities::types::Entity
        .select()
        .all(db).await
}
