use crate::models_enums::videos::*;
use crate::models_enums::types::*;
use sea_orm_migration::prelude::*;

#[derive(DeriveMigrationName)]
pub struct Migration;

#[async_trait::async_trait]
impl MigrationTrait for Migration {
    async fn up(&self, manager: &SchemaManager) -> Result<(), DbErr> {
        // Replace the sample below with your own migration scripts
        manager
            .create_table(
                Table::create()
                    .table(Videos::Table)
                    .if_not_exists()
                    .col(
                        ColumnDef::new(Videos::Id)
                            .integer()
                            .not_null()
                            .auto_increment()
                            .primary_key(),
                    )
                    .col(ColumnDef::new(Videos::Title).string().not_null())
                    .col(ColumnDef::new(Videos::TypeId).integer().not_null())
                    .col(ColumnDef::new(Videos::Url).string().not_null())
                    .col(ColumnDef::new(Videos::AccessCount).integer().default(0))
                    .col(ColumnDef::new(Videos::Duration).integer().default(0))
                    .foreign_key(
                        ForeignKeyCreateStatement::new()
                            .name("fk_types_id")
                            .from(Videos::Table, Videos::TypeId)
                            .to(Types::Table, Types::Id)
                    )
                    .to_owned(),
            )
            .await
    }

    async fn down(&self, manager: &SchemaManager) -> Result<(), DbErr> {
        // Replace the sample below with your own migration scripts

        manager
            .drop_table(Table::drop().table(Videos::Table).to_owned())
            .await
    }
}

