use std::{thread::sleep, time, env};
use redis;

use shit_post_db::{entities::*,*};
use telegram_bot::{TelegramBot, telegram_types::TelegramKeyboardMarkup};

use tera::Tera;

use poem::listener::TcpListener;
use poem::{get, post, Result, Route, Server, EndpointExt};

use shit_post_db::DatabaseConnection;
use shit_post_web_api;

#[tokio::main]
async fn run_t_bot_pull_mode() -> Result<(), ()>{

    let db_url = env::var("DATABASE_URL").expect("DATABASE_URL is not set in .env file");
    let bot_token = env::var("BOT_TOKEN").expect("BOT_TOKEN is not set in .env file");

    //let tg_bot_db_conn: DatabaseConnection = Database::connect(&db_url).await.unwrap();
    let app_db_conn: DatabaseConnection = Database::connect(&db_url).await.unwrap();

    let redis_client: redis::Client = redis::Client::open("redis://127.0.0.1/").unwrap();
    let types: Vec<types::Model> = types::Entity
        .select()
        .all(&app_db_conn).await.unwrap();

    let kb_markup = TelegramKeyboardMarkup{
        keyboard: vec![
            types.clone().into_iter().map(
                |model|  format!("{:}",model.title)
            ).collect()
        ],
        resize_keyboard: true,
    };

    let tg_bot = TelegramBot::new(
        bot_token, redis_client
    );

    'updates_loop: loop {
        sleep(time::Duration::from_secs(2));

        let updates = tg_bot.get_updates().await;
        println!("Data -> | {:?} |", updates);

        'handling_updates: for update in match updates {
            Ok(updates) => updates,
            Err(error_) => {println!("{:?}", error_); continue 'updates_loop}
        } {

            if tg_bot.check_if_handled(update.update_id).unwrap() {
                continue;
            }

            println!("Data -> | {:?} |", update);



            let type_model = types.clone().into_iter().filter(
                |elem| elem.title.contains(&update.message.text)
            ).next();

            let type_model = match type_model {
                Some(model) => model,
                None => {
                    match tg_bot.mark_as_handled(
                        update.update_id
                    ) {
                        Ok(_) => (),
                        Err(_) => panic!("Redis write error, halting via panic!")
                    }
                    continue 'handling_updates;
                }
            };

            let shitpost = match get_random_video_entry_of_type(
                &app_db_conn, type_model.id).await
                {
                    Ok(val) => val,
                    Err(_) => continue 'handling_updates
                };

            match tg_bot.send_message(
                update.message.chat.id, &shitpost.url, &kb_markup
            ).await {
                Ok(_) => (),
                Err(_) => continue 'handling_updates
            };

            match tg_bot.mark_as_handled(update.update_id) {
                Ok(_) => (),
                Err(_) => panic!("Redis write error, halting via panic!")
            }
        }
    }
}



#[tokio::main]
async fn start() -> std::io::Result<()> {
    std::env::set_var("RUST_LOG", "debug");
    tracing_subscriber::fmt::init();

    // get env vars
    //dotenvy::dotenv().ok();
    let bot_token = env::var("BOT_TOKEN").expect("BOT_TOKEN is not set in .env file");

    let db_url = env::var("DATABASE_URL").expect("DATABASE_URL is not set in .env file");
    let server_url: String = env::var("SERVER_URL").expect("SERVER_URL is not set in .env file");

    // create post table if not exists
    let conn: DatabaseConnection = Database::connect(&db_url).await.unwrap();
    let templates = Tera::new(
        &env::var("TERA_TEMPLATES"
    ).expect(
        "TERA_TEMPLATES is not set in .env file"
    )).unwrap();

    let redis_client = redis::Client::open("redis://127.0.0.1/").unwrap();


    let state = shit_post_web_api::AppState {
        conn, templates, redis_client: redis_client.clone(), bot_token: bot_token.clone()
    };

    //TelegramBot::new(bot_token, redis_client).update_webhook().await.unwrap();
    //println!("Starting server at {server_url}");

    let app = Route::new()
    .at("/", get(shit_post_web_api::index))
    .at("/shitpost/add", get(shit_post_web_api::add_shitpost_view))
    .at("/api/shitpost/add", post(shit_post_web_api::post_shitpost))
    .at("/shitpost/:id", get(shit_post_web_api::get_video))
    .at("/shitpost/types", get(shit_post_web_api::get_shitpost_types))
    .at("/web/shitpost/", get(shit_post_web_api::shitpost_view))
    //.at("/bot/", post(shit_post_web_api::tg_bot_handler))
    .data(state);


    let server = Server::new(TcpListener::bind(server_url));
    server.run(app).await
}

fn main() {
    let use_webhook: bool = env::var(
        "USE_WEBHOOK"
    ).expect("USE_WEBHOOK is not set in .env file").parse().expect(
        "USE_WEBHOOK shoud be boolean like"
)   ;

    let _ = env::var(
        "WEBHOOK_ADDR"
    ).expect("WEBHOOK_ADDR  is not set in .env file");

    if !use_webhook {
        run_t_bot_pull_mode().unwrap();
    }
    else
    {
        start().unwrap();
    }

}
